<?php
require_once((defined("RHACO_DIR")?constant("RHACO_DIR"):"")."lang/StringUtil.php");
/**
 * @author Kazutaka Tokushima
 * @license New BSD License
 * @copyright Copyright 2005- The Rhacophorus Project. All rights reserved.
 * @version 0.2.8
 */
class TemplateFormatter{
	function text2Html($value){
		return TemplateFormatter::nl2br(TemplateFormatter::escape($value));
	}
	function escape($value){
		if(!empty($value) && is_string($value)){
			$value = str_replace(array("<",">","'","\""),array("&lt;","&gt;","&#039;","&quot;"),$value);		
		}
		return $value;
	}
	function htmlencode($value){
		if(!empty($value) && is_string($value)){
			$value = htmlentities(StringUtil::encoding($value,StringUtil::getEncodeTypeUTF8()),ENT_QUOTES,StringUtil::getEncodeTypeUTF8());
		}
		return $value;
	}
	function htmldecode($value){
		if(!empty($value) && is_string($value)){
			if(substr(phpversion(),0,1) >= 5){
				$value	= html_entity_decode(StringUtil::encoding($value,StringUtil::getEncodeTypeUTF8()),ENT_QUOTES,StringUtil::getEncodeTypeUTF8());
			}else{
				$value	= StringUtil::encoding($value,StringUtil::getEncodeTypeUTF8());
				$value	= preg_replace('~&#x([0-9a-f]+);~ei','chr(hexdec("\\1"))', $value);
				$value	= preg_replace('~&#([0-9]+);~e', 'chr(\\1)', $value);
				$value	= strtr($value,array_flip(get_html_translation_table(HTML_ENTITIES)));
			}
		}
		return $value;
	}
	function nl2br($value){
		if(!empty($value) && is_string($value)){
			$value = nl2br($value);
		}
		return $value;
	}
	function urlencode($value){
		if(!empty($value) && is_string($value)){
			$value = rawurlencode($value);
		}
		return $value;
	}
	function getJsDocument($value){
		if(!empty($value) && is_string($value)){
			$value = preg_replace("/([\"\'])/","\\\\\\1",$value);
			$value = str_replace("\n","\\\\n",$value);
			$value = str_replace("\r","\\\\r",$value);
		}
		return $value;
	}	
	function httpBuildQuery(){
		$argList = func_get_args();		

		if(is_array($argList[0])){
			$list = $argList[0];
			
			if(sizeof($argList) > 1){
				$list = array();
				
				for($i=1;$i<sizeof($argList);$i++){
					$list[$argList[$i]] = $argList[0][$argList[$i]];
				}
			}
			foreach($list as $key => $request){
				if(!is_object($request) && !is_array($request)){
					$requestString .= sprintf("&%s=%s",$key,urlencode($request));
				}else if(is_array($request)){
					foreach($request as $id => $value){
						$requestString .= sprintf("&%s[%s]=%s", $key,$id,urlencode($value));
					}
				}
			}
			$requestString = substr($requestString,1);
		}
		return $requestString;
	}
	function substring($value,$limit,$offset=0){
		$sep = "";
		
		if(StringUtil::strlen($value) > ($limit - $offset)){
			$sep = "...";
		}
		return StringUtil::substring($value,$offset,$limit).$sep;
	}
	function subexplode($value,$separator,$num=0){
		$list = explode($separator,$value,$num+1);
		
		if(sizeof($list) > $num){
			return $list[$num];
		}
		return "";
	}
	function pregquote($value){
		return preg_quote($value);
	}
	function evenodd($number){
		return (($number % 2) == 0) ? "even" : "odd";
	}
	function pagingQuery($number,$string=""){
		if(!empty($string)){
			$string	= (substr($string,-1) != "&") ? $string."&" : $string;
		}
		$string	= (substr($string,0,1) != "?") ? "?".$string : $string;
		$string	= preg_replace("/[\&\?]page=[\d]+/","",$string);
		return $string."page=".$number;
	}
	function getCdata($value){
		if(preg_match("/<!\[CDATA\[(.+)\]\]>/is",$value,$match)){
			return $match[1];
		}
		return $value;
	}
	
}
?>